# Multiple Object Tracking: Task implementation and ACT-R cognitive model
This repo contains code for running an ACT-R-based implementation of a multiple object tracking task, including a cognitive model that can perform the task. Data can be saved in either or both of two formats for further analysis.

## Files in this repository
This repo contains five files of interest. Each is described briefly below.

### 1) mot_agi.py
This python file runs the simulation, loads and runs the ACT-R model, and saves data. The heart of this implementation is the Experiment() class, which does all of the heavy lifting. A "sweep" function facilitates running multiple instances of the experiment to A) identify ideal model parameters, B) run through different experimental conditions, and C) generate a sufficient sample size for statistical analysis.

### 2) MOT-model.lisp
This is the cognitive model, written in LISP and implemented in ACT-R 7-14, that performs the task defined in mot_agi.py.

### 3) mot_agi_API.docx
Detailed descriptions of everything in mot_agi.py

### 4) requirements.txt
Dependencies: python packages that may need ot be installed for things to work correctly

### 5) README.md
This readme file

### 6) Sweep.ipynb
A Jupyter notebook to make it easy to run everything from one location with no programming necessary

## Usage
There are three ways to use this software:

### 1) Using the Jupyter notebook
Just set the parameters you want and run it (using Jupyter Lab, VS Code, or any other platform capable of running Jupyter code)

### 2) Command line
The "sweep" function can run without any arguments. You can also set parameters as you see fit
```
import mot_agi
mot_agi.sweep()
```

### 3) Command line - single experiment
If you'd like to run a single simulation, you can create and run an instance of the Experiment() class from the command line.
```
import mot_agi
ex = mot_agi.Experiment()
ex.run()
```